/*
 *  Ear Trainer
 * 	By Moses Huang (mgh44) and Brian Cheng (bc295)
 */
#include <avr/eeprom.h>
#include <avr/io.h>
#include <util/delay.h>
#include <stdbool.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <stdint.h>
#include <stddef.h>
#include <inttypes.h>
#include <stdio.h>
#include <math.h>
#include "LCD.h"
#include "lcd_driver.h"

unsigned char isRight; //for relative pitch training, is the cursor on the left or right
unsigned char rightAnswer; //MIDI interrupt, is the answer right or wrong
unsigned char sustain; //bypass state machine
unsigned char keyPressed; //check for multiple button pushes
unsigned char velocity_array[2]; //velocity (not needed)
unsigned char status,note; //stores midi bytes
unsigned char current_byte; //byte currently being read in the rxc interrupt
//char arrays for transforming scores into string
char s[5];
char c[5];

//pointer that keeps track of relative pitch venu
char *step[] = {(char *)zeroStep, (char *)oneStep, (char *)twoStep, (char *)threeStep, (char *)fourStep, (char *)fiveStep, (char *)sixStep, (char *)sevenStep,
                      (char *)eightStep, (char *)nineStep, (char *)tenStep, (char *)elevenStep, (char *)twelveStep};
// The DDS variables 
volatile unsigned int accumulator, accumulator2, accumulator3 ;
volatile u8 highbyte, highbyte2, highbyte3 ;
volatile unsigned int increment, increment2, increment3;

// tables for DDS			
signed char sineTable[256] ;
unsigned char rampTable[256] ;

//linear feedback shift register variables for random generator
uint16_t lfsr = 0xACE1u;
unsigned bit;

//state machine
volatile u8 time1;

//DDS Variable
volatile unsigned int time, sample, rampCount;

//variable for making the sin table
unsigned int i;

//variables for push buttons (debounce, make sure not multiple pressed)
u8 butnum, keyPressed, PushState, samePush;

unsigned int dummy,dummy2, hasClicked; //for higher level of perfect pitch to keep notes pressed
unsigned int position; //which note/choice we are at
unsigned int isBlack; //is it a black note?
unsigned int mode; //keys/replay/exit
unsigned int screen; //which screen we are at
unsigned int testQuestion; //the question
unsigned int testQuestion2, testQuestion3; //quesiton 2 for higher level
u8 questionStatus; //the status of the question
u8 play; //is the play button being pushed?
u8 started; //did we start the training yet? (has the button been pushed)?
unsigned int score; //score
signed char streak; //streak of the user
signed char level; //the current level
//frequencies tables
unsigned int freqtbl[7] = { 274, 308, 346, 366, 411, 461, 518};
unsigned int freqtbl2[7] = {291, 326, 0, 388, 435, 489, 0};
unsigned int freqtbl3[12] = {274, 308, 346, 366, 411, 461, 518, 
							 291, 326, 388, 435, 489};
unsigned int freqtblR[12] = {274, 291, 308, 326, 346, 366, 388, 411, 435, 461, 489, 518};
//tables for relaitve pitch
unsigned int freqtblZero[24] = {C, CSHARP, D, DSHARP, E, F, FSHARP, G, GSHARP, A, ASHARP, B, C, CSHARP, D, DSHARP, E, F, FSHARP, G, GSHARP, A, ASHARP, B};
unsigned int freqtblOne[24] =  {C, CSHARP, D, DSHARP, E, F, FSHARP, G, GSHARP, A, ASHARP, B, CSHARP, D, DSHARP, E, F, FSHARP ,G ,GSHARP ,A ,ASHARP ,B, C+C};
unsigned int freqtblTwo[24] = {C, CSHARP, D, DSHARP, E, F, FSHARP, G, GSHARP, A, ASHARP, B,D, DSHARP, E, F, FSHARP, G, GSHARP, A, ASHARP, B, C+C, CSHARP+CSHARP};
unsigned int freqtblThree[24] = {C, CSHARP, D, DSHARP, E, F, FSHARP, G, GSHARP, A, ASHARP, B, DSHARP, E, F, FSHARP, G, GSHARP, A, ASHARP, B, C+C, CSHARP+CSHARP, D+D};
unsigned int freqtblFour[24] = {C, CSHARP, D, DSHARP, E, F, FSHARP, G, GSHARP, A, ASHARP, B,E, F, FSHARP, G, GSHARP, A, ASHARP, B, C+C, CSHARP+CSHARP, D+D, DSHARP+DSHARP};
unsigned int freqtblFive[24] = {C, CSHARP, D, DSHARP, E, F, FSHARP, G, GSHARP, A, ASHARP, B,F, FSHARP, G, GSHARP, A, ASHARP, B, C+C, CSHARP+CSHARP, D+D, DSHARP+DSHARP, E+E};
unsigned int freqtblSix[24] = {C, CSHARP, D, DSHARP, E, F, FSHARP, G, GSHARP, A, ASHARP, B,FSHARP, G, GSHARP, A, ASHARP, B, C+C, CSHARP+CSHARP, D+D, DSHARP+DSHARP, E+E, F+F};
unsigned int freqtblSeven[24] = {C, CSHARP, D, DSHARP, E, F, FSHARP, G, GSHARP, A, ASHARP, B,G, GSHARP, A, ASHARP, B, C+C, CSHARP+CSHARP,D+D, DSHARP+DSHARP, E+E, F+F, FSHARP+FSHARP};
unsigned int freqtblEight[24] = {C, CSHARP, D, DSHARP, E, F, FSHARP, G, GSHARP, A, ASHARP, B,GSHARP, A, ASHARP, B, C+C, CSHARP+CSHARP,D+D, DSHARP+DSHARP, E+E, F+F, FSHARP+FSHARP, G+G};
unsigned int freqtblNine[24] = {C, CSHARP, D, DSHARP, E, F, FSHARP, G, GSHARP, A, ASHARP, B,A, ASHARP, B, C+C, CSHARP+CSHARP,D+D, DSHARP+DSHARP, E+E, F+F, FSHARP+FSHARP, G+G,GSHARP+GSHARP};
unsigned int freqtblTen[24] = {C, CSHARP, D, DSHARP, E, F, FSHARP, G, GSHARP, A, ASHARP, B,ASHARP, B, C+C, CSHARP+CSHARP,D+D, DSHARP+DSHARP, E+E, F+F, FSHARP+FSHARP, G+G,GSHARP+GSHARP,A+A};
unsigned int freqtblEleven[24] = {C, CSHARP, D, DSHARP, E, F, FSHARP, G, GSHARP, A, ASHARP, B,B, C+C, CSHARP+CSHARP,D+D, DSHARP+DSHARP, E+E, F+F, FSHARP+FSHARP, G+G,GSHARP+GSHARP,A+A, ASHARP+ASHARP};
unsigned int freqtblTwelve[24] = {C, CSHARP, D, DSHARP, E, F, FSHARP, G, GSHARP, A, ASHARP, B,C+C, CSHARP+CSHARP, D+D, DSHARP+DSHARP, E+E, F+F, FSHARP+FSHARP, G+G, GSHARP+GSHARP, A+A, ASHARP+ASHARP, B+B};
//array pointer to all the relative pitch table
unsigned int *RP[] = {(unsigned int *)freqtblZero,(unsigned int *)freqtblOne,(unsigned int *)freqtblTwo,(unsigned int *)freqtblThree,(unsigned int *)freqtblFour,
					(unsigned int *)freqtblFive,(unsigned int *)freqtblSix,(unsigned int *)freqtblSeven,(unsigned int *)freqtblEight,(unsigned int *)freqtblNine,
					 (unsigned int *)freqtblTen,(unsigned int *)freqtblEleven,(unsigned int *)freqtblTwelve };
//display answer
char *answer[] = {(char *) note6, (char *) note7, (char *) note8, (char *) note9, (char *) note10, (char *) note11, (char *) note12, (char *) note1, 
                  (char *) note2, (char *) note3, (char *) note4, (char *) note5 };
//Interrupt for USAR recieve
ISR(USART0_RX_vect) {
	
	switch (current_byte)
	{
	case status_byte:
		status = UDR0;
		//80 is a stop byte, this was for synthesizer, but not used
		if((status & 0xF0) == 0x80) {
			current_byte = status_byte;
			play = 1;
			sustain = 0;
		}
		//90 is note on move to get note
		else if(((status & 0xF0) == 0x90)) 
		{
			current_byte = note_byte;
		}
		
	break;
	case note_byte: 
		
		note = UDR0;
		if((note & 0xF0) == 0x80) {
			play = 1;
			sustain = 0;
			current_byte = status_byte;
		}
		else if((note & 0xF0) == 0x90)
			current_byte = note_byte;
		else {
			current_byte = vel_byte;
			//note is note mod 12
			//i use 24 here because thats the lowest midi key i have			
			note = note - 0x24;
			while (note > 11) {
				note = note - 12; 
			}
			//if there is a question for perfect pitch, check if the answer is right
			if (started) {
				//three notes
				if (level == 2) {
					//if none has been clicked store it into dummy
					if (hasClicked == 0) {
						LCDPutString(sIncorrect, 29, 20, WHITE, WHITE);
						dummy = note;
						hasClicked = 1;
					}
					//if one has been clicked stored it into a second dummy
					else if (hasClicked == 1) {
						hasClicked = 2;
						dummy2 = note;
					}
					//otherwise check if the answer is correct
					else {
						hasClicked = 0;
						if (((testQuestion == convertPosition2(note)) && (testQuestion2 == convertPosition2(dummy)) && (testQuestion3 == convertPosition2(dummy2)))
						     || ((testQuestion == convertPosition2(note)) && (testQuestion2 == convertPosition2(dummy2)) && (testQuestion3 == convertPosition2(dummy)))
							 || ((testQuestion == convertPosition2(dummy)) && (testQuestion2 == convertPosition2(note)) && (testQuestion3 == convertPosition2(dummy2)))
							 || ((testQuestion == convertPosition2(dummy)) && (testQuestion2 == convertPosition2(dummy2)) && (testQuestion3 == convertPosition2(note)))
							 || ((testQuestion == convertPosition2(dummy2)) && (testQuestion2 == convertPosition2(dummy)) && (testQuestion3 == convertPosition2(note)))
							 || ((testQuestion == convertPosition2(dummy2)) && (testQuestion2 == convertPosition2(note)) && (testQuestion3 == convertPosition2(dummy)))
			     		  ) {
								questionStatus = 0;
								rightAnswer = 1;
						}
						else {
							rightAnswer = 2;
						}
					}
				}
				//two notes
				else if (level == 1) {
					//if none has been clicked store into a dummy
					if (hasClicked == 0) {
						LCDPutString(sIncorrect, 29, 20, WHITE, WHITE);
						dummy = note;
						hasClicked = 1;
					}
					//else check if the answer is correct
					else {
						hasClicked = 0;
						if (((testQuestion == convertPosition2(note)) && (testQuestion2 == convertPosition2(dummy)))
					    || ((testQuestion == convertPosition2(dummy)) && (testQuestion2 == convertPosition2(note))) ) {
							questionStatus = 0;
							rightAnswer = 1;
						}
						else {
							rightAnswer = 2;
						}
					}
				}
				//one note, check if the answer is correct
				else if (level == 0) {
					if (testQuestion == convertPosition2(note)) {
						questionStatus = 0;
						rightAnswer = 1;
					}
					else {
						rightAnswer = 2;
					}
				}
			}
			//uncomment for synthesizer to work
			if (screen == 3) {
					TIMSK0 = (1<<TOIE0);
					TCCR0A = (1<<COM0A0) | (1<<COM0A1) | (1<<WGM00) | (1<<WGM01) ;
					sample = 0;
					rampCount = 0;
					increment = freqtblR[note];
					increment2 = freqtblR[note];
					increment3 = freqtblR[note];
					sustain = 1;
			}
		}

	break;
	//this was in case we need the velocity byte, but we did not use it.
	case vel_byte: //read the 3rd byte of a message
		velocity_array[0] = UDR0;
		if((velocity_array[0] & 0xF0) == 0x80) {
			current_byte = status_byte;
			play = 1;
			sustain = 0;
		}
		else
			current_byte = note_byte;
		break;
	}
	
}

//ISR for state machine
ISR(TIMER2_COMPA_vect)
{
	if (time1 > 0) --time1;
}
//ISR for DDS from lab2
ISR (TIMER0_OVF_vect)
{
	accumulator = accumulator + increment;
	accumulator2 = accumulator2 + increment2;
	accumulator3 = accumulator3 + increment3;

	highbyte = (char)(accumulator >> 8) ;
	highbyte2 = (char)(accumulator2 >> 8);
	highbyte3 = (char)(accumulator3 >> 8);

	OCR0A = 128 + (((sineTable[highbyte] + sineTable[highbyte2]+sineTable[highbyte3]) * rampTable[rampCount])>>7) ;
	
	if (sample <= RAMPUPEND) {
		sample++ ;
		rampCount++ ;
	}
	else if (sample > RAMPUPEND && sample <= RAMPDOWNSTART2 ) {
		sample++ ;
		rampCount = 255 ;
	}
	//sustain = 1 when MIDI bypass state machine
	//STOP CLOCK and reset variable
	else if (PushState == Released)
	{
		sample++ ;
		rampCount-- ;
		if (sample > RAMPDOWNEND) {
			rampCount = 0; 
   			sample = 0;
			accumulator = 0;
			accumulator2 = 0;
			accumulator3 =0;
			increment = 0;
			increment2 = 0;
			increment3 = 0;
			TCCR0A = 0;
			TIMSK0 = 0;
  		}
	}
}
//Draws the keyboard for perfect pitch training/synthesizer
void DrawKeyboard(void) {
	
	LCDPutString(sExit, 10, 90, BROWN, WHITE);
	LCDSetRect(7, 87, 21, 115, 0, BLACK);
	//if it is not synthesizer, draw level, score
	if (screen != 3) {
		LCDPutString(sReplay, 26, 90, GREEN, WHITE);
		LCDSetRect(23, 87, 37, 115, 0, BLACK);
		LCDPutString(sLevel, 5, 5, RED, WHITE);
		itoa(level+1, c, 10);
		LCDPutString(c, 5, 40, RED, WHITE);
		LCDPutString(sProgress, 13, 5, BLUE, WHITE);
		itoa(score, c, 10);
		LCDPutString(c, 21, 5, BLUE,WHITE);
	}
	//if it is not relative pitch training, draw the keyboard
	if (screen != 2) {
		LCDSetRect(65, dim[0], 130, dim[1], 0, BLACK); //19  C
		LCDSetRect(66, dim[0]+1, 129, dim[1]-1, 1, YELLOW); //19 
		LCDPutChar('C', 55, dim[0]+7, BLUE, WHITE);

		LCDSetRect(65, dim[1], 130, dim[2], 0, BLACK);//18 D
		LCDPutChar('D', 55, dim[1]+7, BLUE, WHITE);

		LCDSetRect(65, dim[2], 130, dim[3], 0, BLACK);//19 E
		LCDPutChar('E', 55, dim[2]+7, BLUE, WHITE);

		LCDSetRect(65, dim[3], 130, dim[4], 0, BLACK); //18 F
		LCDPutChar('F', 55, dim[3]+7, BLUE, WHITE);

		LCDSetRect(65, dim[4], 130, dim[5], 0, BLACK); //19 G
		LCDPutChar('G', 55, dim[4]+7, BLUE, WHITE);

		LCDSetRect(65, dim[5], 130, dim[6], 0, BLACK); //18 A
		LCDPutChar('A', 55, dim[5]+7, BLUE, WHITE);

		LCDSetRect(65, dim[6], 130, dim[7], 0, BLACK); //19 B
		LCDPutChar('B', 55, dim[6]+7, BLUE, WHITE);

		LCDSetRect(65, 17, 105, 21, 1, BLACK); //19 C#
		LCDPutString(note1, 46, dim2[0]-5, BLUE, WHITE);

		LCDSetRect(65, 35, 105, 39, 1, BLACK); //19 D#
		LCDPutString(note2, 46, dim2[1]-5, BLUE, WHITE);

		LCDSetRect(65, 72, 105, 76, 1, BLACK); //19 F#
		LCDPutString(note3, 46, dim2[3]-5, BLUE, WHITE);

		LCDSetRect(65, 91, 105, 95, 1, BLACK); //19 G#
		LCDPutString(note4, 46, dim2[4]-5, BLUE, WHITE);

		LCDSetRect(65, 109, 105, 113, 1, BLACK); //19 A#
		LCDPutString(note5, 46, dim2[5]-5, BLUE, WHITE);
	}
	//if it is relative pitch training, draw the multiple choice menu
	if (screen == 2) {
		LCDPutString((char *)step[0], 46, 1, BLACK, YELLOW);
		LCDPutString((char *)step[2], 58, 1, BLACK, WHITE);
		LCDPutString((char *)step[4], 70, 1, BLACK, WHITE);
		LCDPutString((char *)step[6], 82, 1, BLACK, WHITE);
		LCDPutString((char *)step[8], 94, 1, BLACK, WHITE);
		LCDPutString((char *)step[10], 106, 1, BLACK, WHITE);
		LCDPutString((char *)step[12], 118, 1, BLACK, WHITE);

		LCDPutString((char *)step[1], 46, 65, BLACK, WHITE);
		LCDPutString((char *)step[3], 58, 65, BLACK, WHITE);
		LCDPutString((char *)step[5], 70, 65, BLACK, WHITE);
		LCDPutString((char *)step[7], 82, 65, BLACK, WHITE);
		LCDPutString((char *)step[9], 94, 65, BLACK, WHITE);
		LCDPutString((char *)step[11], 106, 65, BLACK, WHITE);
		

	}
}
//Changes the position of the multiple choice menu (highlight)
//based on what button is pressed and the current highlighted choice
void ChangeMC(void) {
	if (butnum == UP) {
		//if it is at play, move to exit
		if (mode == REPLAY) {
			LCDSetRect(24, 88, 36, 114, 1, WHITE);
			LCDPutString(sReplay, 26, 90, GREEN, WHITE);
			LCDSetRect(8, 88, 20, 114, 1, YELLOW);
			LCDPutString(sExit, 10, 90, BROWN, YELLOW);
			mode = EXIT;
		}
		else if (mode == KEY) {
			//if it is at the top of the choices, move to play
			if(position == 0 || position == 1) {
				if (isRight)
					LCDPutString((char *)step[position], dim3[0], 65, BLACK, WHITE);
				else
					LCDPutString((char *)step[position], dim3[0], 1, BLACK, WHITE);
					mode = REPLAY; //set mode to replay
					LCDSetRect(24, 88, 36, 114, 1, YELLOW);
					LCDPutString(sReplay, 26, 90, GREEN, YELLOW);
			}
			else {
				//move up by subtracting by 2 (since there are two per row)
				if (isRight) {
					LCDPutString((char *)step[position], dim3[position], 65, BLACK, WHITE);
					position = position - 2;
					LCDPutString((char *)step[position], dim3[position], 65, BLACK, YELLOW);
				}
				else {
					LCDPutString((char *)step[position], dim3[position], 1, BLACK, WHITE);
					position = position - 2;
					LCDPutString((char *)step[position], dim3[position], 1, BLACK, YELLOW);
				}
			}
		}
		else {
			//otherwise move from exit to the choices
			LCDSetRect(8, 88, 20, 114, 1, WHITE);
			LCDPutString(sExit, 10, 90, BROWN, WHITE);
			mode = KEY;
			position = 12;
			isRight = 0;
			LCDPutString((char *)step[12], dim3[position], 1, BLACK, YELLOW);
			
		}
	} 
	else if (butnum == DOWN) {
		//if it is at exit, move to replay
		if (mode == EXIT) {
				LCDSetRect(8, 88, 20, 114, 1, WHITE);
				LCDPutString(sExit, 10, 90, BROWN, WHITE);
				LCDSetRect(24, 88, 36, 114, 1, YELLOW);
				LCDPutString(sReplay, 26, 90, GREEN, YELLOW);
				mode = REPLAY;
		}
		else if (mode == KEY) {
			//if it is at bottom move to EXIT
			if (position == 11 || position == 12) {
				if (isRight) 
					LCDPutString((char *)step[position], dim3[position], 65, BLACK, WHITE);
				else
					LCDPutString((char *)step[position], dim3[position], 1, BLACK, WHITE);
				LCDSetRect(8, 88, 20, 114, 1, YELLOW);
				LCDPutString(sExit, 10, 90, BROWN, YELLOW);
				mode = EXIT;
			}
			else {
				//move down by add by 2 (since there are two per row)
				if (isRight) {
					LCDPutString((char *)step[position], dim3[position], 65, BLACK, WHITE);
					position = position + 2;
					LCDPutString((char *)step[position], dim3[position], 65, BLACK, YELLOW);
				}
				else {
					LCDPutString((char *)step[position], dim3[position], 1, BLACK, WHITE);
					position = position + 2;
					LCDPutString((char *)step[position], dim3[position], 1, BLACK, YELLOW);
				}
				
			}
		}
		else {
			//move from play to the choices
			LCDSetRect(24, 88, 36, 114, 1, WHITE);
			LCDPutString(sReplay, 26, 90, GREEN, WHITE);
			mode = KEY;
			position = 0;
			LCDPutString((char *)step[0], dim3[position], 1, BLACK, YELLOW);
			isRight = 0;
		}
	}
	else {
		//if it is a choice on the right, highlight the left one
		if (position == 1 || position == 3 || position == 5 || position == 7 || position == 9 || position == 11) {
			LCDPutString((char *)step[position], dim3[position], 65, BLACK, WHITE);
			position = position -1;
			LCDPutString((char *)step[position], dim3[position], 1, BLACK, YELLOW);
			isRight = 0;
		}
		//otherwise highlight the right one
		else if (position != 12) {
			LCDPutString((char *)step[position], dim3[position], 1, BLACK, WHITE);
			position = position +1;
			LCDPutString((char *)step[position], dim3[position], 65, BLACK, YELLOW);
			isRight = 1;
		}
	}
}
//highlight different keys in perfect pitch training based on the button pushed and the current position
void ChangeKeyboard(int direction, int current, int black) {
		if (direction == UP) {
			// if it is at play move to exit
			if (mode == REPLAY) {
				LCDSetRect(24, 88, 36, 114, 1, WHITE);
				LCDPutString(sReplay, 26, 90, GREEN, WHITE);
				LCDSetRect(8, 88, 20, 114, 1, YELLOW);
				LCDPutString(sExit, 10, 90, BROWN, YELLOW);
				mode = EXIT;
			//if at keyboard
			}else if (mode == KEY) {
				//if it is at a black key unhighlight the key
				if (black == 1) {
				LCDSetRect(65, dim2[current], 105, dim2[current]+4, 1, BLACK);
				}
				//special case for unhighlighting if it is at right side of the keyboard
				else if (current == 6) {
					LCDSetRect(66, dim[current]+1, 129, dim[current+1]-1, 1, WHITE);
					LCDSetRect(65, dim2[current-1], 105, dim2[current-1]+4, 1, BLACK);
				}
				//check if the white key has a black key next to it
				else if (dim2[current] == 0) {
					LCDSetRect(66, dim[current]+1, 129, dim[current+1]-1, 1, WHITE); //19 
					LCDSetRect(65, dim2[current-1], 105, dim2[current-1]+4, 1, BLACK); //19 C#
				}
				else {
					LCDSetRect(66, dim[current]+1, 129, dim[current+1]-1, 1, WHITE); //19 		
					if (dim2[current-1] != 0) {
						LCDSetRect(65, dim2[current-1], 105, dim2[current-1]+4, 1, BLACK); //19 C#
					}
					LCDSetRect(65, dim2[current], 105, dim2[current]+4, 1, BLACK); //19 C#
				}
				//if at perfect pitch training, change the mode to play
				if (screen == 1) {
					mode = REPLAY; //set mode to replay
					LCDSetRect(24, 88, 36, 114, 1, YELLOW);
					LCDPutString(sReplay, 26, 90, GREEN, YELLOW);
				}
				//if not, change the mode to exit
				else {
					mode = EXIT;
					LCDSetRect(8, 88, 20, 114, 1, YELLOW);
					LCDPutString(sExit, 10, 90, BROWN, YELLOW);
					}
			}else {
				//otherwise the highlighting and unhighlighting is the same
				LCDSetRect(8, 88, 20, 114, 1, WHITE);
				LCDPutString(sExit, 10, 90, BROWN, WHITE);
				if (isBlack) {
					LCDSetRect(66, dim2[current]+1, 104, dim2[current]+3, 1, YELLOW);
				}
				else {
					LCDSetRect(66, dim[current]+1, 129, dim[current+1]-1, 1, YELLOW);
					if (dim2[current] != 0) {
						LCDSetRect(65, dim2[current], 105, dim2[current]+4, 1, BLACK);
					}
					if (dim2[current-1] != 0) {
						LCDSetRect(65, dim2[current-1], 105, dim2[current-1]+4, 1, BLACK);
					}
				}
				mode = KEY;
			}
			
		}
		//if the down button is pushed
		else if (direction == DOWN) {
			// if it is at exit, move to play
			if (mode == EXIT) {
				LCDSetRect(8, 88, 20, 114, 1, WHITE);
				LCDPutString(sExit, 10, 90, BROWN, WHITE);
				//if at perfect pitch training, move to play
				if (screen == 1) {
					LCDSetRect(24, 88, 36, 114, 1, YELLOW);
					LCDPutString(sReplay, 26, 90, GREEN, YELLOW);
					mode = REPLAY;
				}
				//otherwise move down to the keys
				else {
					mode = KEY;
						if (isBlack) {
					LCDSetRect(66, dim2[current]+1, 104, dim2[current]+3, 1, YELLOW);
				}
				else {
					LCDSetRect(66, dim[current]+1, 129, dim[current+1]-1, 1, YELLOW);
					if (dim2[current] != 0) {
						LCDSetRect(65, dim2[current], 105, dim2[current]+4, 1, BLACK);
					}
					if (dim2[current-1] != 0) {
						LCDSetRect(65, dim2[current-1], 105, dim2[current-1]+4, 1, BLACK);
					}
				}
			}
			//if it is at the keyboard
			}else if (mode == KEY) {
				//check if it is at a black key
				if (black == 1) {
				LCDSetRect(65, dim2[current], 105, dim2[current]+4, 1, BLACK);
				}
				//special case for right side of the keyboard
				else if (current == 6) {
					LCDSetRect(66, dim[current]+1, 129, dim[current+1]-1, 1, WHITE);
					LCDSetRect(65, dim2[current-1], 105, dim2[current-1]+4, 1, BLACK);
				}
				else if (dim2[current] == 0) {
					LCDSetRect(66, dim[current]+1, 129, dim[current+1]-1, 1, WHITE); //19 
					LCDSetRect(65, dim2[current-1], 105, dim2[current-1]+4, 1, BLACK); //19 C#
				}
				else {
					LCDSetRect(66, dim[current]+1, 129, dim[current+1]-1, 1, WHITE); //19 			
					if (dim2[current-1] != 0) {
						LCDSetRect(65, dim2[current-1], 105, dim2[current-1]+4, 1, BLACK); //19 C#
					}
					LCDSetRect(65, dim2[current], 105, dim2[current]+4, 1, BLACK); //19 C#
				}
				mode = 2; //set mode to exit
				LCDSetRect(8, 88, 20, 114, 1, YELLOW);
				LCDPutString(sExit, 10, 90, BROWN, YELLOW);
			}else {
					LCDSetRect(24, 88, 36, 114, 1, WHITE);
				LCDPutString(sReplay, 26, 90, GREEN, WHITE);
				if (isBlack) {
					LCDSetRect(66, dim2[current]+1, 104, dim2[current]+3, 1, YELLOW);
				}
				else {
					LCDSetRect(66, dim[current]+1, 129, dim[current+1]-1, 1, YELLOW);
					if (dim2[current] != 0) {
						LCDSetRect(65, dim2[current], 105, dim2[current]+4, 1, BLACK);
					}
					if (dim2[current-1] != 0) {
						LCDSetRect(65, dim2[current-1], 105, dim2[current-1]+4, 1, BLACK);
					}
				}
				mode = KEY; //set mode to key
			}
			
		}
		//if right button is pushed and at keyboard (doesn't do anything if it is at play or exit)
		//highlight and unhighlight accordingly
		else if (direction == RIGHT && mode == KEY) {
			if (black == 0) {
				if (current == 6) {
					LCDSetRect(66, dim[current]+1, 129, dim[current+1]-1, 1, WHITE); //19 
					LCDSetRect(65, dim2[current-1], 105, dim2[current-1]+4, 1, BLACK); //19 C#
					LCDSetRect(66, dim[0]+1, 129, dim[1]-1, 1, YELLOW); //19 
					LCDSetRect(65, dim2[0], 105, dim2[0]+4, 1, BLACK); //19 C#
					position = 0;
				}
				else if (dim2[current] == 0) {
					LCDSetRect(66, dim[current]+1, 129, dim[current+1]-1, 1, WHITE); //19 
					LCDSetRect(65, dim2[current-1], 105, dim2[current-1]+4, 1, BLACK); //19 C#
					LCDSetRect(66, dim[current+1]+1, 129, dim[current+2]-1, 1, YELLOW); //19 
					LCDSetRect(65, dim2[current+1], 105, dim2[current+1]+4, 1, BLACK); //19 C#
					position++;
				}
				else {
					LCDSetRect(66, dim[current]+1, 129, dim[current+1]-1, 1, WHITE); //19 		
					if (dim2[current-1] != 0) {
						LCDSetRect(65, dim2[current-1], 105, dim2[current-1]+4, 1, BLACK); //19 C#
					}
					LCDSetRect(65, dim2[current], 105, dim2[current]+4, 1, BLACK); //19 C#
					LCDSetRect(66, dim2[current]+1, 104, dim2[current]+3, 1, YELLOW); //19 C#
					isBlack = 1;
				}
			}
			//else for if it is black
			else {
				LCDSetRect(66, dim[current+1]+1, 129, dim[current+2]-1, 1, YELLOW); //19 
				LCDSetRect(65, dim2[current], 105, dim2[current]+4, 1, BLACK); //19 C#
				if (dim2[current+1] != 0) {
					LCDSetRect(65, dim2[current+1], 105, dim2[current+1]+4, 1, BLACK);
				}
				position++;
				isBlack = 0;
			}
		}
		else {
		//for the left button now
		if (mode == KEY) {
			if (black == 0) {
				if (current == 0) {
					LCDSetRect(66, dim[current]+1, 129, dim[current+1]-1, 1, WHITE); //19 
					LCDSetRect(65, dim2[current], 105, dim2[current]+4, 1, BLACK); //19 C#
					LCDSetRect(66, dim[6]+1, 129, dim[7]-1, 1, YELLOW); //19 
					LCDSetRect(65, dim2[5], 105, dim2[5]+4, 1, BLACK); //19 C#
					position = 6;
				}
				else if (current == 3) {
					LCDSetRect(66, dim[current]+1, 129, dim[current+1]-1, 1, WHITE); //19 
					LCDSetRect(65, dim2[current], 105, dim2[current]+4, 1, BLACK); //19 C#
					LCDSetRect(66, dim[current-1]+1, 129, dim[current]-1, 1, YELLOW); //19 
					LCDSetRect(65, dim2[current-2], 105, dim2[current-2]+4, 1, BLACK); //19 C#
					position--;
				}
				else {
					LCDSetRect(66, dim[current]+1, 129, dim[current+1]-1, 1, WHITE); //19 		
					if (dim2[current] != 0) {
						LCDSetRect(65, dim2[current], 105, dim2[current]+4, 1, BLACK); //19 C#
					}
					LCDSetRect(65, dim2[current-1], 105, dim2[current-1]+4, 1, BLACK); //19 C#
					LCDSetRect(66, dim2[current-1]+1, 104, dim2[current-1]+3, 1, YELLOW); //19 C#
					isBlack = 1;
					position--;
				}
			}
			//else for if it is black
			else {
				LCDSetRect(66, dim[current]+1, 129, dim[current+1]-1, 1, YELLOW); //19 
				LCDSetRect(65, dim2[current], 105, dim2[current]+4, 1, BLACK); //19 C#
				if (dim2[current-1] != 0) {
					LCDSetRect(65, dim2[current-1], 105, dim2[current-1]+4, 1, BLACK);
				}
				isBlack = 0;
			}
		}
		}
	
}

//state machine
void task1(void)
{
	time1 = t1;
	switch (PushState)
	{
		//released state
		case Released:
			//check if there is an MIDI input which bypasses the state machine when it is at released state
			if (rightAnswer == 1) {
				right();
				rightAnswer = 0;
			}
			else if (rightAnswer == 2) {
				wrong();
				rightAnswer = 0;
			}
			//scan for push buttons
			scan();
			if(butnum > 0 && butnum < 6){
					PushState = Debounce;
					samePush = butnum;
				}
			break;
		case Debounce:
			//scan for push buttons
			scan();
			//if it is the same go to Pushed otherwise go to Released
			if (butnum > 0 && butnum < 6 && butnum == samePush)
					PushState = Pushed;
			else PushState = Released;
			
			break;
		case Pushed:
			//if Enter is Pushed
			if  (butnum == ENTER) {
				//If exit, clear everything and draws welcome screen and reset variables
				if (mode == EXIT){
					LCDClearScreen();
					WelcomeScreen();
					position = 0;
					mode = 0;
					screen = 0;
					started = 0;
					questionStatus = 0;
					isBlack = 0;
					score = 0;
					streak = 0;
					level = 0;
					dummy = 0;
					hasClicked = 0;
				}
			//if it is in relative pitch
			else if (screen == 2) {
				// if play is pressed, generate random number
				if (mode == REPLAY) {
					if (questionStatus == 0) {
						LCDPutString(sIncorrect, 29, 20, WHITE, WHITE);
						started = 1;
						questionStatus = 1;
						random();
						eeprom_write_word((uint16_t*)46, lfsr);
					}
					//frequencies based on the random generator
					increment = ((unsigned int*) RP[testQuestion])[testQuestion2];
					increment2 = ((unsigned int*)RP[testQuestion])[testQuestion2+12];
					//turn on clock and move to stillpushed state
					TIMSK0 = (1<<TOIE0);
					TCCR0A = (1<<COM0A0) | (1<<COM0A1) | (1<<WGM00) | (1<<WGM01) ;
					PushState = StillPushed;
				}
				//if there is already a question, check if the answer is right and update score accordingly
				else if (mode == KEY && started) {
					if (testQuestion == position) {
						questionStatus = 0;
						LCDPutString(sIncorrect, 29, 20, WHITE, WHITE);
						LCDPutString(sCorrect, 29, 20, GREEN, WHITE);
						itoa(score, c, 10);
						if (streak < 1)
							score++;
						else if (streak == 1) 
							score += 2;
						else if (streak == 2)
							score += 5;
						else if (streak > 2)
							score += 10;
						streak++;
						LCDPutString(c, 21, 5, WHITE,WHITE);
						itoa(score, c, 10);
						LCDPutString(c, 21, 5, BLUE,WHITE);
						started = 0;
					}
					else {
						LCDPutString(sIncorrect, 29, 20, RED, WHITE);
						streak = 0;
						if (score > 0) {
							itoa(score, c, 10);
							LCDPutString(c, 21, 5, WHITE,WHITE);
							score--;
							itoa(score, c, 10);
							LCDPutString(c, 21, 5, BLUE,WHITE);
						}
					}
				}
			}
			//perfect pitch
			else if (screen == 1) {
				//if play is pressed, generate random numbers
				if (mode == REPLAY) {
					if (questionStatus == 0) {
						LCDPutString(sIncorrect, 29, 20, WHITE, WHITE);
						started = 1;
						questionStatus = 1;
						random();
						eeprom_write_word((uint16_t*)46, lfsr);
					}
					//frequencies based on note(s)
					increment = freqtbl3[testQuestion];
					if (level == 0) {
						increment2 = freqtbl3[testQuestion];
						increment3 = freqtbl3[testQuestion];
					}
					else if (level == 1){
						increment2 = freqtbl3[testQuestion2];
					}
					else {
						increment2 = freqtbl3[testQuestion2];
						increment3 = freqtbl3[testQuestion3];
					}
					//turn on clock and go to stillPushed
						TIMSK0 = (1<<TOIE0);
						TCCR0A = (1<<COM0A0) | (1<<COM0A1) | (1<<WGM00) | (1<<WGM01) ;
						PushState = StillPushed;
					}
					//if a key is pressed, play that sound
					else if (mode == KEY) {
						if (level == 0) {
							TIMSK0 = (1<<TOIE0);
							TCCR0A = (1<<COM0A0) | (1<<COM0A1) | (1<<WGM00) | (1<<WGM01) ;
							if (!isBlack){
								sample = 0;
								rampCount = 0;
								increment = freqtbl[position];
								increment2 = freqtbl[position];
								increment3 = freqtbl[position];
							}
							else {
								sample = 0;
								rampCount = 0;
								increment = freqtbl2[position];
								increment2 = freqtbl2[position];
								increment3 = freqtbl2[position];
							}
						}
						//if there are two note and question has started
						if (level == 1 && started) {
							//if no note has been chosen, store into dummy
							if (hasClicked == 0) {
								LCDPutString(sIncorrect, 29, 20, WHITE, WHITE);
								dummy = convertPosition();
								hasClicked = 1;
								//highlight
								if (dummy < 7) {
									LCDPutChar(character[dummy], 55, dim[dummy]+7, RED, YELLOW);
								}
								else {
									if (dummy == 7) {
										LCDPutString(note1, 46, dim2[0]-5, RED, YELLOW);
									}
								else if (dummy == 8) {
									LCDPutString(note2, 46, dim2[1]-5, RED, YELLOW);
								}
								else if (dummy == 9) {
									LCDPutString(note3, 46, dim2[3]-5, RED, YELLOW);
								}
								else if (dummy == 10) {
									LCDPutString(note4, 46, dim2[4]-5, RED, YELLOW);
								}
								else if (dummy == 11) {
									LCDPutString(note5, 46, dim2[5]-5, RED, YELLOW);
								}
							}
						}
						// if one note has been chosen, check if the current one is equal to the stored one
						//if so remove the highlight and change hasClicked
						else {
							if (dummy == convertPosition()) {
								hasClicked = 0;
								if (dummy < 7) {
									LCDPutChar(character[dummy], 55, dim[dummy]+7, BLUE, WHITE);
								}
								else {
									if (dummy == 7) {
										LCDPutString(note1, 46, dim2[0]-5, BLUE, WHITE);
									}
									else if (dummy == 8) {
										LCDPutString(note2, 46, dim2[1]-5, BLUE, WHITE);
									}
									else if (dummy == 9) {
										LCDPutString(note3, 46, dim2[3]-5, BLUE, WHITE);
									}
									else if (dummy == 10) {
										LCDPutString(note4, 46, dim2[4]-5, BLUE, WHITE);
									}
									else if (dummy == 11) {
										LCDPutString(note5, 46, dim2[5]-5, BLUE, WHITE);
									}
								}
							}
						else {
							//otherwise start clock and play the two notes
							TIMSK0 = (1<<TOIE0);
							TCCR0A = (1<<COM0A0) | (1<<COM0A1) | (1<<WGM00) | (1<<WGM01) ;											
							sample = 0;
							rampCount = 0;
							increment = freqtbl3[dummy];
							increment2 = freqtbl3[convertPosition()];
							increment3 = 0;
							hasClicked = 0;
							if (dummy < 7) {
								LCDPutChar(character[dummy], 55, dim[dummy]+7, BLUE, WHITE);
							}
							else {
								if (dummy == 7) {
									LCDPutString(note1, 46, dim2[0]-5, BLUE, WHITE);
								}
								else if (dummy == 8) {
									LCDPutString(note2, 46, dim2[1]-5,BLUE, WHITE);
								}
								else if (dummy == 9) {
									LCDPutString(note3, 46, dim2[3]-5,BLUE, WHITE);
								}
								else if (dummy == 10) {
									LCDPutString(note4, 46, dim2[4]-5,BLUE, WHITE);
								}
								else if (dummy == 11) {
									LCDPutString(note5, 46, dim2[5]-5, BLUE, WHITE);
								}
							}
							//check to see if it is the right answer and update score accordingly
							if (((testQuestion == convertPosition()) && (testQuestion2 == dummy))
							    || ((testQuestion == dummy) && (testQuestion2 == convertPosition())) ) {
								questionStatus = 0;
								LCDPutString(sIncorrect, 29, 20, WHITE, WHITE);
								LCDPutString(sCorrect, 29, 20, GREEN, WHITE);
								itoa(score, c, 10);
								if (streak < 1)
									score++;
								else if (streak == 1) 
									score += 2;
								else if (streak == 2)
									score += 5;
								else if (streak > 2)
									score += 10;
								streak++;
								LCDPutString(c, 21, 5, WHITE,WHITE);
								itoa(score, c, 10);
								LCDPutString(c, 21, 5, BLUE,WHITE);
								started = 0;
								//level up
								if (score > 29) {
									level = 2;
									itoa(level+1, c, 10);
									LCDPutString(c, 5, 40, RED, WHITE);
								}
							}
							else {
								LCDPutString(sIncorrect, 29, 20, RED, WHITE);
								streak = 0;
								if (score > 0) {
									itoa(score, c, 10);
									LCDPutString(c, 21, 5, WHITE,WHITE);
									score--;
									itoa(score, c, 10);
									LCDPutString(c, 21, 5, BLUE,WHITE);
								}
								//level down
								if (score < 10) {
									level = 0;
									itoa(level+1, c, 10);
									LCDPutString(c, 5, 40, RED, WHITE);
									hasClicked = 0;
									started = 0;
									questionStatus = 0;
								}
							}
						}
					}
				}
				//three notes
				else if (level == 2 && started) {
					//if no notes has been stored, store it into dummy and highlight
					if (hasClicked == 0) {
						LCDPutString(sIncorrect, 29, 20, WHITE, WHITE);
						dummy = convertPosition();
						hasClicked = 1;
						if (dummy < 7) {
							LCDPutChar(character[dummy], 55, dim[dummy]+7, RED, YELLOW);
						}
						else {
							if (dummy == 7) {
								LCDPutString(note1, 46, dim2[0]-5, RED, YELLOW);
							}
							else if (dummy == 8) {
								LCDPutString(note2, 46, dim2[1]-5, RED, YELLOW);
							}
							else if (dummy == 9) {
								LCDPutString(note3, 46, dim2[3]-5, RED, YELLOW);
							}
							else if (dummy == 10) {
								LCDPutString(note4, 46, dim2[4]-5, RED, YELLOW);
							}
							else if (dummy == 11) {
								LCDPutString(note5, 46, dim2[5]-5, RED, YELLOW);
							}
						}
					}
					//if one note has been chosen, check the current note to see if it is the same as the one before
					//if so, remove highlight.
					else if (hasClicked == 1) {
						if (dummy == convertPosition()) {
							hasClicked = 0;
							if (dummy < 7) {
								LCDPutChar(character[dummy], 55, dim[dummy]+7, BLUE, WHITE);
							}
							else {
								if (dummy == 7) {
									LCDPutString(note1, 46, dim2[0]-5, BLUE, WHITE);
								}
							else if (dummy == 8) {
								LCDPutString(note2, 46, dim2[1]-5, BLUE, WHITE);
							}
							else if (dummy == 9) {
								LCDPutString(note3, 46, dim2[3]-5, BLUE, WHITE);
							}
							else if (dummy == 10) {
								LCDPutString(note4, 46, dim2[4]-5, BLUE, WHITE);
							}
							else if (dummy == 11) {
								LCDPutString(note5, 46, dim2[5]-5, BLUE, WHITE);
							}
						}
					}
					else {
					//otherwise, store the note to a second dummy variable and highlight
						hasClicked = 2;
						dummy2 = convertPosition();
						if (dummy2 < 7) {
							LCDPutChar(character[dummy2], 55, dim[dummy2]+7, RED, YELLOW);
						}
						else {
							if (dummy2 == 7) {
								LCDPutString(note1, 46, dim2[0]-5, RED, YELLOW);
							}
							else if (dummy2 == 8) {
								LCDPutString(note2, 46, dim2[1]-5, RED, YELLOW);
							}
							else if (dummy2 == 9) {
								LCDPutString(note3, 46, dim2[3]-5, RED, YELLOW);
							}
							else if (dummy2 == 10) {
								LCDPutString(note4, 46, dim2[4]-5, RED, YELLOW);
							}
							else if (dummy2 == 11) {
								LCDPutString(note5, 46, dim2[5]-5, RED, YELLOW);
							}
						}
					}
				}
				//otherwise check if the third note is the same as any of the previous one
				//and remove highlight if so
				else {
					if (dummy == convertPosition()) {
						hasClicked = 1;
						if (dummy < 7) {
							LCDPutChar(character[dummy], 55, dim[dummy]+7, BLUE, WHITE);
						}
						else {
							if (dummy == 7) {
								LCDPutString(note1, 46, dim2[0]-5, BLUE, WHITE);
							}
							else if (dummy == 8) {
								LCDPutString(note2, 46, dim2[1]-5, BLUE, WHITE);
							}
							else if (dummy == 9) {
								LCDPutString(note3, 46, dim2[3]-5, BLUE, WHITE);
							}
							else if (dummy == 10) {
								LCDPutString(note4, 46, dim2[4]-5, BLUE, WHITE);
							}
							else if (dummy == 11) {
								LCDPutString(note5, 46, dim2[5]-5, BLUE, WHITE);
							}
						}
					}
					else if (dummy2 == convertPosition()) {
						hasClicked = 1;
						if (dummy2 < 7) {
							LCDPutChar(character[dummy2], 55, dim[dummy2]+7, BLUE, WHITE);
						}
						else {
							if (dummy2 == 7) {
								LCDPutString(note1, 46, dim2[0]-5, BLUE, WHITE);
							}
							else if (dummy2 == 8) {
								LCDPutString(note2, 46, dim2[1]-5, BLUE, WHITE);
							}
							else if (dummy2 == 9) {
								LCDPutString(note3, 46, dim2[3]-5, BLUE, WHITE);
							}
							else if (dummy2 == 10) {
								LCDPutString(note4, 46, dim2[4]-5, BLUE, WHITE);
							}
							else if (dummy2 == 11) {
								LCDPutString(note5, 46, dim2[5]-5, BLUE, WHITE);
							}
						}
					}
					else {
						//turn on clock and play the three notes chosen
						TIMSK0 = (1<<TOIE0);
						TCCR0A = (1<<COM0A0) | (1<<COM0A1) | (1<<WGM00) | (1<<WGM01) ;											
						sample = 0;
						rampCount = 0;
						increment = freqtbl3[dummy];
						increment2 = freqtbl3[convertPosition()];
						increment3 = freqtbl3[dummy2];
						hasClicked = 0;
						//remove highlight
						if (dummy < 7) {
							LCDPutChar(character[dummy], 55, dim[dummy]+7, BLUE, WHITE);
						}
						else {
							if (dummy == 7) {
								LCDPutString(note1, 46, dim2[0]-5, BLUE, WHITE);
							}
							else if (dummy == 8) {
								LCDPutString(note2, 46, dim2[1]-5,BLUE, WHITE);
							}
							else if (dummy == 9) {
								LCDPutString(note3, 46, dim2[3]-5,BLUE, WHITE);
							}
							else if (dummy == 10) {
								LCDPutString(note4, 46, dim2[4]-5,BLUE, WHITE);
							}
							else if (dummy == 11) {
								LCDPutString(note5, 46, dim2[5]-5, BLUE, WHITE);
							}
						}
						if (dummy2 < 7) {
							LCDPutChar(character[dummy2], 55, dim[dummy2]+7, BLUE, WHITE);
						}
						else {
							if (dummy2 == 7) {
								LCDPutString(note1, 46, dim2[0]-5, BLUE, WHITE);
							}
							else if (dummy2 == 8) {
								LCDPutString(note2, 46, dim2[1]-5,BLUE, WHITE);
							}
							else if (dummy2 == 9) {
								LCDPutString(note3, 46, dim2[3]-5,BLUE, WHITE);
							}
							else if (dummy2 == 10) {
								LCDPutString(note4, 46, dim2[4]-5,BLUE, WHITE);
							}
							else if (dummy2 == 11) {
								LCDPutString(note5, 46, dim2[5]-5, BLUE, WHITE);
							}
						}
						//check to see if the answer is right, and update score accordingly
						if (((testQuestion == convertPosition()) && (testQuestion2 == dummy) && (testQuestion3 == dummy2))
						     || ((testQuestion == convertPosition()) && (testQuestion2 == dummy2) && (testQuestion3 == dummy))
							 || ((testQuestion == dummy) && (testQuestion2 == convertPosition()) && (testQuestion3 == dummy2))
							 || ((testQuestion == dummy) && (testQuestion2 == dummy2) && (testQuestion3 == convertPosition()))
							 || ((testQuestion == dummy2) && (testQuestion2 == dummy) && (testQuestion3 == convertPosition()))
							 || ((testQuestion == dummy2) && (testQuestion2 == convertPosition()) && (testQuestion3 == dummy))
							  ) {
							questionStatus = 0;
							LCDPutString(sIncorrect, 29, 20, WHITE, WHITE);
							LCDPutString(sCorrect, 29, 20, GREEN, WHITE);
							itoa(score, c, 10);
							if (streak < 1)
								score++;
							else if (streak == 1) 
								score += 2;
							else if (streak == 2)
								score += 5;
							else if (streak > 2)
								score += 10;
							streak++;
							LCDPutString(c, 21, 5, WHITE,WHITE);
							itoa(score, c, 10);
							LCDPutString(c, 21, 5, BLUE,WHITE);
							started = 0;
						}
						else {
							LCDPutString(sIncorrect, 29, 20, RED, WHITE);
							streak = 0;
							if (score > 0) {
								itoa(score, c, 10);
								LCDPutString(c, 21, 5, WHITE,WHITE);
								score--;
								itoa(score, c, 10);
								LCDPutString(c, 21, 5, BLUE,WHITE);
							}
							//level down
							if (score < 30) {
								level = 1;
								itoa(level+1, c, 10);
								LCDPutString(c, 5, 40, RED, WHITE);
								hasClicked = 0;
								started = 0;
								questionStatus = 0;
							}
						}
					}
				}
			}
			//one note
			else if (started) {
			//check if the answer is correct and update score accordingly
				if (testQuestion == convertPosition()) {
					questionStatus = 0;
					LCDPutString(sIncorrect, 29, 20, WHITE, WHITE);
					LCDPutString(sCorrect, 29, 20, GREEN, WHITE);
					itoa(score, c, 10);
					if (streak < 1)
						score++;
					else if (streak == 1) 
						score += 2;
					else if (streak == 2)
						score += 5;
					else if (streak > 2)
						score += 10;
					streak++;
					LCDPutString(c, 21, 5, WHITE,WHITE);
					itoa(score, c, 10);
					LCDPutString(c, 21, 5, BLUE,WHITE);
					started = 0;
					if (score > 9) {
						level = 1;
						itoa(level+1, c, 10);
						LCDPutString(c, 5, 40, RED, WHITE);
					}
				}
				else {
					LCDPutString(sIncorrect, 29, 20, RED, WHITE);
					streak = 0;
					if (score > 0) {
						itoa(score, c, 10);
						LCDPutString(c, 21, 5, WHITE,WHITE);
						score--;
						itoa(score, c, 10);
						LCDPutString(c, 21, 5, BLUE,WHITE);
					}
					if (score < 10) {
						level = 0;
						itoa(level+1, c, 10);
						LCDPutString(c, 5, 40, RED, WHITE);
					}
				}
			}
		}
	}
	//welcome screen, and update accordingly to which one is chosen
	else if (screen == 0) {
		if (position == 0) {
			screen = 1;
		}
		else if (position == 1) {
			screen = 2;
		}
		else
			screen = 3;
		position = 0;
		LCDClearScreen();
		DrawKeyboard();
	}
	else {
			//synthesizer mode play notes
		 if (mode == KEY && screen == 3) {
			if (!isBlack){
				TIMSK0 = (1<<TOIE0);
				TCCR0A = (1<<COM0A0) | (1<<COM0A1) | (1<<WGM00) | (1<<WGM01) ;
				sample = 0;
				rampCount = 0;
				increment = freqtbl[position];
				increment2 = freqtbl[position];
			}
			else {
				TIMSK0 = (1<<TOIE0);
				TCCR0A = (1<<COM0A0) | (1<<COM0A1) | (1<<WGM00) | (1<<WGM01) ;
				sample = 0;
				rampCount = 0;
				increment = freqtbl2[position];
				increment2 = freqtbl2[position];
			}
		}
	}
}	//left button pressed
	else if (butnum == LEFT) {
		//if relative pitch, change the highlight
		if (screen == 2) {
			ChangeMC();
		}
		//if perfect pitch/synthesizer, change the highlight
		else if (screen != 0) {
			ChangeKeyboard(butnum, position, isBlack);
		}
	}
	//up is pressed
	else if (butnum == UP) {
		//update welcome screen.
		if (screen == 0) {
			if (position == 0) {
				LCDPutString (perfectPitch, 50, 5, RED, WHITE);
				LCDPutString (synthesizer, 70, 5, BLUE, YELLOW);
				position = 2;
			}
			else if (position == 1) {
				LCDPutString (perfectPitch, 50, 5,RED, YELLOW);
				LCDPutString (relativePitch, 60, 5, GREEN, WHITE);
				position = 0;
			}
			else {
				LCDPutString (relativePitch, 60, 5, GREEN, YELLOW);
				LCDPutString (synthesizer, 70, 5, BLUE, WHITE);
				position = 1;
			}
		}
		//update relative pitch screen
		else if (screen == 2) {
			ChangeMC();
		}
		//update perfect pitch/synthesizer
		else {
			ChangeKeyboard(butnum, position, isBlack);
		}
	}
	//down is pressed
	else if (butnum == DOWN) {
		//update welcome screen
		if (screen == 0) {
			if (position == 0) {
				LCDPutString (perfectPitch, 50, 5, RED, WHITE);
				LCDPutString (relativePitch, 60, 5, GREEN, YELLOW);
				position = 1;
			}
			else if (position == 1) {
				LCDPutString (synthesizer, 70, 5, BLUE, YELLOW);
				LCDPutString (relativePitch, 60, 5, GREEN, WHITE);
				position = 2;
			}
			else {
				LCDPutString (perfectPitch, 50, 5, RED, YELLOW);
				LCDPutString (synthesizer, 70, 5, BLUE, WHITE);
				position = 0;
			}
		}
		//update relative pitch
		else if (screen == 2) {
			ChangeMC();
		}
		//update perfect pitch/synthesizer
		else {
			ChangeKeyboard(butnum, position, isBlack);
		}
	}
	//right is pressed
	else if (butnum == RIGHT) {
		//update relative pitch
		if (screen == 2) {
			ChangeMC();
		}
		//update perfect pitch/synthesizer
		else if (screen != 0) {
			ChangeKeyboard(butnum, position, isBlack);
		}
	}
	PushState = StillPushed;
	break;
	case StillPushed:
		//scan to see if the button is the same
		scan();
		if (!(butnum > 0 && butnum < 6 && butnum == samePush))
			PushState = MaybeReleased;
		break;

	case MaybeReleased:
		//scan to see if the button is the same
		scan();
		if (butnum > 0 && butnum < 6 && butnum == samePush)
			PushState = StillPushed;
		else {
			PushState = Released;
			sample = RAMPDOWNSTART;
		}
	break;
	}
}
//convert positions since my numbering of keys is inconsistent
int convertPosition2(int x) {
	switch (x) {
		case 0: return 0; break;
		case 1: return 7; break;
		case 2: return 1; break;
		case 3: return 8; break;
		case 4: return 2; break;
		case 5: return 3; break;
		case 6: return 9; break;
		case 7: return 4; break;
		case 8: return 10; break;
		case 9: return 5;break;
		case 10: return 11; break;
		case 11: return 6; break;
	}
}
//convert positions since my numbering of keys is inconsistent
int convertPosition(void) {
		if (!isBlack) 
			return position;
		else {
			switch (position) {
				case 0: return 7; break;
				case 1: return 8; break;
				case 3: return 9; break;
				case 4: return 10; break;
				case 5: return 11; break;
			}
		}
}
//scan for buttons pressed.
//if more than three buttons are pressed the answer will be shown.

void scan(void)
{
	if (~PINA & 0x01) {
		butnum = ENTER;
		keyPressed++;
	}
	if(~PINA & 0x02) {
		butnum = LEFT; 
		keyPressed++;
	}
	if(~PINA & 0x04) {
		butnum = UP; 
		keyPressed++;
	}
	if(~PINA & 0x08) {
		butnum = DOWN; 
		keyPressed++;
	}
	if(~PINA & 0x10) {
		butnum = RIGHT;
		keyPressed++;
	}
	if (keyPressed != 1) {
		butnum = 0;
		itoa(testQuestion, c, 10);
		//erase answer
		if (screen == 1) {
			LCDPutString((char *)answer[testQuestion], 21, 30, WHITE, WHITE);
			LCDPutString((char *)answer[testQuestion2], 21, 45, WHITE, WHITE);
			LCDPutString((char *)answer[testQuestion3], 21, 60, WHITE, WHITE);
		}
		else if (screen == 2) {
			itoa(testQuestion, c, 10);
			LCDPutString(c, 21, 30, WHITE,WHITE);
		}
	}
	if (keyPressed > 2) {
		//show answers
		if (screen == 2) {
			itoa(testQuestion, c, 10);
			LCDPutString(c, 21, 30, BLUE,WHITE);
		}
		else if (screen == 1) {
			LCDPutString((char *)answer[testQuestion], 21, 30, BLUE, WHITE);
			if (level > 0) {
				LCDPutString((char *)answer[testQuestion2], 21, 45, BLUE, WHITE);
			}
			if (level > 1) {
				LCDPutString((char *)answer[testQuestion3], 21, 60, BLUE, WHITE);
			}
		}
		
	
	}
	keyPressed = 0;
}
//random generator
void random(void) {
	
	int temp = testQuestion;
	testQuestion = 16;
	if (screen == 1) {
	//keep generating until the random number is within range and is different from last one
	while (testQuestion > 11 || testQuestion == temp) {
		//taps
		bit  = ((lfsr >> 0) ^ (lfsr >> 2) ^ (lfsr >> 3) ^ (lfsr >> 5) ) & 1;
    	lfsr =  (lfsr >> 1) | (bit << 15);
		testQuestion = 0; 
		if (lfsr & 0x0001) {
			testQuestion++;
		}
		if (lfsr & 0x0002) {
			testQuestion += 2;
		}
		if (lfsr & 0x0004) {
			testQuestion += 4;
		}
		if (lfsr & 0x0008) {
			testQuestion += 8;
		}
	
	}
	//generate another one for two notes
	if (level > 0) {
	testQuestion2 = 16;
		while (testQuestion2 > 11 || testQuestion2 == testQuestion) {
			bit  = ((lfsr >> 0) ^ (lfsr >> 2) ^ (lfsr >> 3) ^ (lfsr >> 5) ) & 1;
    		lfsr =  (lfsr >> 1) | (bit << 15);
			testQuestion2 = 0; 
			if (lfsr & 0x0001) {
				testQuestion2++;
			}
			if (lfsr & 0x0002) {
				testQuestion2 += 2;
			}
			if (lfsr & 0x0004) {
				testQuestion2 += 4;
			}
			if (lfsr & 0x0008) {
				testQuestion2 += 8;
			}
			
	
		}
	}
	//generate yet another one for three notes
	if (level > 1) {
	testQuestion3 = 16;
		while (testQuestion3 > 11 || testQuestion3 == testQuestion || testQuestion3 == testQuestion2) {
			bit  = ((lfsr >> 0) ^ (lfsr >> 2) ^ (lfsr >> 3) ^ (lfsr >> 5) ) & 1;
    		lfsr =  (lfsr >> 1) | (bit << 15);
			testQuestion3 = 0; 
			if (lfsr & 0x0001) {
				testQuestion3++;
			}
			if (lfsr & 0x0002) {
				testQuestion3 += 2;
			}
			if (lfsr & 0x0004) {
				testQuestion3 += 4;
			}
			if (lfsr & 0x0008) {
				testQuestion3 += 8;
			}
			
	
		}
	}}
	//for relative pitch, generate two random numbers for interval and note
	else {
		while (testQuestion > 12 || testQuestion == temp) {
		bit  = ((lfsr >> 0) ^ (lfsr >> 2) ^ (lfsr >> 3) ^ (lfsr >> 5) ) & 1;
    	lfsr =  (lfsr >> 1) | (bit << 15);
		testQuestion = 0; 
		if (lfsr & 0x0001) {
			testQuestion++;
		}
		if (lfsr & 0x0002) {
			testQuestion += 2;
		}
		if (lfsr & 0x0004) {
			testQuestion += 4;
		}
		if (lfsr & 0x0008) {
			testQuestion += 8;
		}
	
		}
		testQuestion2 = 16;
		while (testQuestion2 > 11 || testQuestion2 == testQuestion) {
			bit  = ((lfsr >> 0) ^ (lfsr >> 2) ^ (lfsr >> 3) ^ (lfsr >> 5) ) & 1;
    		lfsr =  (lfsr >> 1) | (bit << 15);
			testQuestion2 = 0; 
			if (lfsr & 0x0001) {
				testQuestion2++;
			}
			if (lfsr & 0x0002) {
				testQuestion2 += 2;
			}
			if (lfsr & 0x0004) {
				testQuestion2 += 4;
			}
			if (lfsr & 0x0008) {
				testQuestion2 += 8;
			}
			
	
		}
	}

	
}
//update score and play sound if MIDI input is correct
void right(void) {
	TIMSK0 = (1<<TOIE0);
	TCCR0A = (1<<COM0A0) | (1<<COM0A1) | (1<<WGM00) | (1<<WGM01) ;
	sample = 0;
	rampCount = 0;
	if (level == 1) {
		increment = freqtblR[dummy];
		increment2 = freqtblR[note];
		increment3 = 0;
	}
	else if(level == 0) {
		increment = freqtblR[note];
		increment2 = freqtblR[note];
		increment3 = freqtblR[note];
	}
	else if (level == 2) {
		increment = freqtblR[dummy];
	increment2 = freqtblR[note];
	increment3 = freqtblR[dummy2];
	}
	play = 1;
	LCDPutString(sIncorrect, 29, 20, WHITE, WHITE);
	LCDPutString(sCorrect, 29, 20, GREEN, WHITE);
	itoa(score, c, 10);
	if (streak < 1)
		score++;
	else if (streak == 1) 
		score += 2;
	else if (streak == 2)
		score += 5;
	else if (streak > 2)
		score += 10;
	streak++;
	
	LCDPutString(c, 21, 5, WHITE,WHITE);
	itoa(score, c, 10);
	LCDPutString(c, 21, 5, BLUE,WHITE);
	started = 0;
	if (level == 1) {
		if (score > 29) {
			level = 2;
			itoa(level+1, c, 10);
			LCDPutString(c, 5, 40, RED, WHITE);
		}	
	}
	else if (level == 0) {
		if (score > 9) {
			level = 1;
			itoa(level+1, c, 10);
			LCDPutString(c, 5, 40, RED, WHITE);
		}	
	}
}
//update score and play sound if MIDI input is wrong
void wrong(void) {
	LCDPutString(sIncorrect, 29, 20, RED, WHITE);
	TIMSK0 = (1<<TOIE0);
	TCCR0A = (1<<COM0A0) | (1<<COM0A1) | (1<<WGM00) | (1<<WGM01) ;
	sample = 0;
	rampCount = 0;
	if (level == 1) {
		increment = freqtblR[dummy];
		increment2 = freqtblR[note];
		increment3 = 0;
	}
	else if(level == 0) {
		increment = freqtblR[note];
		increment2 = freqtblR[note];
		increment3 = freqtblR[note];
	}
	else if (level == 2) {
		increment = freqtblR[dummy];
		increment2 = freqtblR[note];
		increment3 = freqtblR[dummy2];
	}
	play = 1;
	streak = 0;
	if (score > 0) {
		itoa(score, c, 10);
		LCDPutString(c, 21, 5, WHITE,WHITE);
		score--;
		itoa(score, c, 10);
		LCDPutString(c, 21, 5, BLUE,WHITE);
	}
	if (level == 1) {
		if (score < 10) {
			level = 0;
			itoa(level+1, c, 10);
			LCDPutString(c, 5, 40, RED, WHITE);
			hasClicked = 0;
			started = 0;
			questionStatus = 0;
		}
	}
	if (level == 2) {
		if (score < 30) {
			level = 1;
			itoa(level+1, c, 10);
			LCDPutString(c, 5, 40, RED, WHITE);
			hasClicked = 0;
			started = 0;
			questionStatus = 0;
		}
	}
}
int main()
{
	//read the seed for linear feedback random generator
	uint16_t temp = eeprom_read_word((uint16_t*)46);
	//if no record, use a chosen seed, otherwise use the one stored in eeprom
	if (temp == 0) {
		lfsr = 0xACE1u;
	}
	else
		lfsr = temp;

  //set up timer 0 for 1 mSec timebase 
  OCR2A = 249;  		//set the compare re to 250 time ticks
  TIMSK2= (1<<OCIE0A);	//turn on timer 0 cmp match ISR 
  //set prescalar to divide by 64 
  TCCR2B= 3; //0b00000011;	
  // turn on clear-on-match
  TCCR2A= (1<<WGM01) ;
  //initializes uart
  uart_init();
   DDRB = (1 <<PINB3) | (1 << PINB0); //B.3 is an output
   DDRA = 0xC0;	// PORT A is an input
	//initialization
  time1 = t1;
  PushState = Released;
  //clock turned off initially to save resource
  TCCR0B = 1;  
  TIMSK0 = 0;
  TCCR0A = 0;

  OCR0A = 128 ;
  //create sine table
	for (i=0; i<256; i++)
   {
		sineTable[i] = (char)(42.3 * sin(6.283*((float)i)/256.0)) ; //127.0
		rampTable[i] = i>>1 ;
   }  
	//variables for optimal contrast for LCD
	int bright = 42;
	int resistance = 3;
	// Enable the IO lines
	PORTD |= 0xE0;
	DDRD = 0xE0;	

	// LCD setup
	init_lcd();
	LCDClearScreen();
	//setup LCD brightness
	lcd_set_brightness_parms(bright, resistance);
	//Draws the welcome screen
	WelcomeScreen();

    //turn on interrupt
		sei();
		//infinite loop running the statemachine task
		while(1) {
			if (time1 == 0) {
				task1();
			}
		}

	return 0;
	
}
