//Draws the Welcome Screen menu
void WelcomeScreen(void) {
	LCDPutString (ear, 5, 5, BLACK, WHITE);
	LCDPutString (mainMenu, 20, 32, BLACK, WHITE);
	LCDSetLine(30, 30, 30, 90, BLACK);
	LCDPutString (perfectPitch, 50, 5, RED, YELLOW);
	LCDPutString (relativePitch, 60, 5, GREEN, WHITE);
	LCDPutString (synthesizer, 70, 5, BLUE, WHITE);
	LCDPutString (cornell, 110, 5, WHITE, BLACK);
	LCDPutString (author, 120, 5, WHITE, BLACK);
}
//initialize uart
void uart_init(void)
{
	//equation from data sheet is (16000000 / (16UL * UART_BAUD)) - 1
	//with UART_BAUD of MIDI = 31250
	UBRR0L = 31;
    //enable rx and receive ISR
	UCSR0B = 0b10010000;
}

//SPI to communicate with the LCD
void sendByte(bool cmd, u8 data)
{
	// clk starts high
	asm("sbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTD)));

	// enable chip_sel
	asm("cbi %0, 7" : : "I" (_SFR_IO_ADDR(PORTD)));
	
	// Send the command flag bit
	asm("cbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTD)));
	asm("cbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTD)));
	asm("sbrc %0, 0" : : "a" (cmd));
	asm("sbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTD)));
	asm("sbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTD)));
	
	// Send Bit 7 of data
	asm("cbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTD)));
	asm("cbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTD)));
	asm("sbrc %0, 7" : : "a" (data));
	asm("sbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTD)));
	asm("sbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTD)));
	
	// Send Bit 6 of data
	asm("cbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTD)));
	asm("cbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTD)));
	asm("sbrc %0, 6" : : "a" (data));
	asm("sbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTD)));
	asm("sbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTD)));
	
	// Send bit 5 of data
	asm("cbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTD)));
	asm("cbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTD)));
	asm("sbrc %0, 5" : : "a" (data));
	asm("sbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTD)));
	asm("sbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTD)));
	
	// Send bit 4 of data
	asm("cbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTD)));
	asm("cbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTD)));
	asm("sbrc %0, 4" : : "a" (data));
	asm("sbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTD)));
	asm("sbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTD)));
	
	// Send bit 3 of data
	asm("cbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTD)));
	asm("cbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTD)));
	asm("sbrc %0, 3" : : "a" (data));
	asm("sbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTD)));
	asm("sbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTD)));
	
	// Send bit 2 of data
	asm("cbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTD)));
	asm("cbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTD)));
	asm("sbrc %0, 2" : : "a" (data));
	asm("sbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTD)));
	asm("sbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTD)));
	
	// Send bit 1 of data
	asm("cbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTD)));
	asm("cbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTD)));
	asm("sbrc %0, 1" : : "a" (data));
	asm("sbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTD)));
	asm("sbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTD)));
		
	// Send bit 0 of data
	asm("cbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTD)));
	asm("cbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTD)));
	asm("sbrc %0, 0" : : "a" (data));
	asm("sbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTD)));
	asm("sbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTD)));
	
	// disable chip_sel
	asm("sbi %0, 7" : : "I" (_SFR_IO_ADDR(PORTD)));
}

//Adjust the contrast of the LCD (manually test by variations)
void lcd_set_brightness_parms(uint8_t a, uint8_t b)
{
	sendByte(lctCmd, VOLCTR);
	sendByte(lctData, a);
	sendByte(lctData, b);

}
//Initializes the LCD
void init_lcd(void)
{
	//RESET
	PORTB &= ~0x1;
	_delay_us(1);
	PORTB |= 0x1;
	_delay_ms(2);
	
	sendByte(lctCmd, DISCTL);	// Display Control
	sendByte(lctData, 0x00);	// default
	sendByte(lctData, 0x20);//from 20	// (32 + 1) * 4 = 132 lines (of which 130 are visible)
	sendByte(lctData, 0x00);	// default, but tutorial says 0x00

	sendByte(lctCmd, COMSCN);	// Common Scan
	sendByte(lctData, 0x01);	// COM1-80: 1->80; COM81-160: 160<-81 (otherwise bottom of screen is upside down)

	sendByte(lctCmd, OSCON);	// Oscillation On
	sendByte(lctCmd, SLPOUT);	// Sleep Out (exit sleep mode)

	sendByte(lctCmd, VOLCTR);	// Electronic Volume Control (brightness/contrast)
	sendByte(lctData, 28);
	sendByte(lctData, 3);

	sendByte(lctCmd, TMPGRD);
	sendByte(lctData, 0);		// default

	sendByte(lctCmd, PWRCTR);	// Power Control Set
	sendByte(lctData, 0x0f);

	sendByte(lctCmd, DISINV);	// Inverse Display
	sendByte(lctCmd, PTLOUT);	// Partial Out (no partial display)

	sendByte(lctCmd, DATCTL);	// Data Control
	sendByte(lctData, 0x01);	// normal orientation; scan across cols, then rows
	sendByte(lctData, 0x00);	// RGB arrangement (RGB all rows/cols)
	sendByte(lctData, 0x02);	// 16 bit-color display

    LCDClearScreen(); //clear screen to white
	_delay_ms(300); //need the delay before ready to display
	sendByte(lctCmd, DISON);	// Display On
}

//Clear Screen of LCD by drawing all WHITE
void LCDClearScreen(void) {
	long i; // loop counter
	// Row address set (command 0x2B)
 	sendByte(lctCmd, PASET);
	 sendByte(lctData, 0);
 	sendByte(lctData, 131);
	// Column address set (command 0x2A)
 	sendByte(lctCmd, CASET);
	 sendByte(lctData, 0);
 	sendByte(lctData, 131);
	// set the display memory to WHITE
 	sendByte(lctCmd, RAMWR);
	for(i = 0; i < ((131 * 131) / 2); i++) {
		sendByte(lctData,(WHITE >> 4) & 0xFF); 
		sendByte(lctData,((WHITE & 0xF) << 4) | ((WHITE >> 8) & 0xF)); 
		sendByte(lctData,WHITE & 0xFF); 
	}
}
//set a color of a specific pixel on the screen
void LCDSetPixel(int  x, int  y, int  color){
	// Row address set  (command 0x2B) 
 	sendByte(lctCmd,PASET); 
 	sendByte(lctData,x); 
 	sendByte(lctData,x); 
	// Column address set  (command 0x2A) 
 	sendByte(lctCmd,CASET); 
 	sendByte(lctData,y); 
 	sendByte(lctData,y); 
   	// Now illuminate the pixel (2nd pixel will be ignored) 
 	sendByte(lctCmd,RAMWR); 
	sendByte(lctData,(color >> 4) & 0xFF); 
	sendByte(lctData,((color & 0xF) << 4) | ((color >> 8) & 0xF)); 
	sendByte(lctData,color & 0xFF);
}
//set a line on the LCD to a color
void LCDSetLine(int x0, int y0, int x1, int y1, int color){
	int dy = y1 - y0; 
  	int dx = x1 - x0; 
  	int stepx, stepy;         
  	if (dy < 0) { dy = -dy;  stepy = -1; } else { stepy = 1; } 
    if (dx < 0) { dx = -dx;  stepx = -1; } else { stepx = 1; } 
    dy <<= 1; // dy is now 2*dy 
    dx <<= 1; // dx is now 2*dx 
  	LCDSetPixel(x0, y0, color); 
    if (dx > dy) { 
    	  int fraction = dy - (dx >> 1); // same as 2*dy - dx 
          while (x0 != x1) { 
      		if (fraction >= 0) { 
      			y0 += stepy; 
				fraction -= dx; // same as fraction -= 2*dx 
 			}                
  			x0 += stepx;
  			fraction += dy; // same as fraction -= 2*dy 
            LCDSetPixel(x0, y0, color);
		}  
  	} else { 
  		int fraction = dx - (dy >> 1);
        while (y0 != y1) {
      		if (fraction >= 0) { 
      			x0 += stepx; 
				fraction -= dy; 
 			}
  			y0 += stepy;
  			fraction += dx; 
  			LCDSetPixel(x0, y0, color); 
 		}
 	}  
}
//Draw a Rectangle on the screen
void LCDSetRect(int x0, int y0, int x1, int y1, unsigned char fill, int color){
	int   xmin, xmax, ymin, ymax; 
 	int i; 
	// check if the rectangle is to be filled 
	if (fill == FILL) { 
     
		// best way to create a filled rectangle is to define a drawing box 
		// and loop two pixels at a time 
     
		// calculate the min and max for x and y directions 
		xmin = (x0 <= x1) ? x0 : x1;
		xmax = (x0 > x1) ? x0 : x1; 
		ymin = (y0 <= y1) ? y0 : y1;
		ymax = (y0 > y1) ? y0 : y1; 
     
		// specify the controller drawing box according to those limits 
		// Row address set  (command 0x2B) 
  		sendByte(lctCmd,PASET); 
  		sendByte(lctData,xmin); 
  		sendByte(lctData,xmax); 
	     
		// Column address set  (command 0x2A) 
  		sendByte(lctCmd,CASET); 
	  	sendByte(lctData,ymin); 
  		sendByte(lctData,ymax); 
     
  		// WRITE MEMORY 
  		sendByte(lctCmd,RAMWR); 
     
		// loop on total number of pixels / 2 
		for (i = 0; i < ((((xmax - xmin + 1) * (ymax - ymin + 1)) / 2) + 130); i++) { 
 			// use the color value to output three data bytes covering two pixels 
   			sendByte(lctData,(color >> 4) & 0xFF); 
			sendByte(lctData,((color & 0xF) << 4) | ((color >> 8) & 0xF)); 
   			sendByte(lctData,color & 0xFF); 
		}    
	} else { 
   	    // best way to draw un unfilled rectangle is to draw four lines 
		LCDSetLine(x0, y0, x1, y0, color); 
		LCDSetLine(x0, y1, x1, y1, color); 
		LCDSetLine(x0, y0, x0, y1, color); 
		LCDSetLine(x1, y0, x1, y1, color); 
 	}
}
//Draw a circle on the screen
void LCDSetCircle(int x0, int y0, int radius, int color){
	
	int f = 1 - radius;
	int ddF_x = 0;
	int ddF_y = -2 * radius; 
	int x = 0; 
	int y = radius; 
  
	LCDSetPixel(x0, y0 + radius, color); 
	LCDSetPixel(x0, y0 - radius, color); 
	LCDSetPixel(x0 + radius, y0, color); 
	LCDSetPixel(x0 - radius, y0, color); 
  
	while(x < y) {
		if(f >= 0) { 
   			y--; 
   			ddF_y += 2;
   			f += ddF_y; 
		}    
  		x++; 
  		ddF_x += 2;
		f += ddF_x + 1;     
		LCDSetPixel(x0 + x, y0 + y, color); 
		LCDSetPixel(x0 - x, y0 + y, color); 
		LCDSetPixel(x0 + x, y0 - y, color); 
		LCDSetPixel(x0 - x, y0 - y, color); 
		LCDSetPixel(x0 + y, y0 + x, color); 
		LCDSetPixel(x0 - y, y0 + x, color); 
		LCDSetPixel(x0 + y, y0 - x, color); 
		LCDSetPixel(x0 - y, y0 - x, color); 
 	}
 }
 //Draw a String on the screen
void LCDPutString (char *pString, int  x, int  y, int fcolor, int bcolor){
    
	// loop until null-terminator is seen 
	while (*pString != 0x00) { 
		// draw the character 
		LCDPutChar(*pString++, x, y, fcolor, bcolor);
		// advance the y position 
		y = y + 6; 
 		// bail out if y exceeds 131
		if (y > 131) break; 
 
	}
}
// Draw a Character on the screen
void LCDPutChar(char c, int x, int y, int fColor, int bColor) {
	extern const unsigned char FONT6x8[97][8];
	int i,j;
	unsigned int nCols;
	unsigned int nRows;
	unsigned int nBytes;
	unsigned char PixelRow;
	unsigned char Mask;
	unsigned int Word0;
	unsigned int Word1;
	unsigned char *pFont;
	unsigned char *pChar;
	unsigned char *FontTable[] = {(unsigned char *)FONT6x8};
	// get pointer to the beginning of the selected font table
	pFont = (unsigned char *)FontTable[0];
	// get the nColumns, nRows and nBytes
	nCols = *pFont;
	nRows = *(pFont + 1);
	nBytes = *(pFont + 2);
	// get pointer to the last byte of the desired character
	pChar = pFont + (nBytes * (c - 0x1F)) + nBytes - 1;
	// Row address set (command 0x2B)
	sendByte(lctCmd, PASET);
	sendByte(lctData,x);
	sendByte(lctData, x + nRows - 1);
	// Column address set (command 0x2A)
	sendByte(lctCmd, CASET);
	sendByte(lctData,y);
	sendByte(lctData, y + nCols - 1);
	// WRITE MEMORY
	sendByte(lctCmd, RAMWR);
	// loop on each row, working backwards from the bottom to the top
	for (i = nRows - 1; i >= 0; i--) {
		// copy pixel row from font table and then decrement row
		PixelRow = *pChar--;
		// loop on each pixel in the row (left to right)
		// Note: we do two pixels each loop
		Mask = 0x80;
		for (j = 0; j < nCols; j += 2) {
		// if pixel bit set, use foreground color; else use the background color
		// now get the pixel color for two successive pixels
			if ((PixelRow & Mask) == 0)
				Word0 = bColor;
			else
				Word0 = fColor;
			Mask = Mask >> 1;
			if ((PixelRow & Mask) == 0)
				Word1 = bColor;
			else
				Word1 = fColor;
			Mask = Mask >> 1;
			// use this information to output three data bytes
			sendByte(lctData, (Word0 >> 4) & 0xFF);
			sendByte(lctData, ((Word0 & 0xF) << 4) | (( Word1 >> 8) & 0xF));
			sendByte(lctData, Word1 & 0xFF);
		}
	}
	// terminate the Write Memory command
	sendByte(lctCmd, NOP);
}
